/****Juan Ma, Oklahoma State University*******/
/*******juan.ma@okstate.edu 1/16/2014********/
libname myLib 'C:\Users\juam\Dropbox\Product Afinity_DBGCJM\affinity_data';

%let inputname = affinity_data;
%let numRecords = 30000;

*get all variable names;
proc contents data = myLib.&inputname order=varnum out=work.Affinity_data_contents; run;

*get product frequency variable names;
data work.Affinity_data_contents_sm; set work.Affinity_data_contents;
     keep name;
     if substr(name,1,10) = 'Prod_Quant';
run;

* get the length of product frequency variable names;
data _null_;
    set work.Affinity_data_contents_sm end = eof;
	retain lname 0;
	lname = max(lname,length(name));
	if eof then put lname=;
run;

*create marco all_name = list of all product frequency variable, and other marco variables like avg_name...;
data _null_;
    set work.Affinity_data_contents_sm end = eof;
    length all_name avg_name std_name max_name min_name sm_name $200;
	retain all_name avg_name std_name max_name min_name sm_name ' ';
  if name ^= ' ' then do;
    all_name = strip(all_name) !! ' ' !! strip(name);  
    avg_name = strip(avg_name) !! ' avg_' !! strip(name);  
    std_name = strip(std_name) !! ' std_' !! strip(name);  
    max_name = strip(max_name) !! ' max_' !! strip(name);  
    min_name = strip(min_name) !! ' min_' !! strip(name);
    sm_name = strip(sm_name) !! ' sm_' !! strip(name);
  end;
    if eof then do;
	   call symput('all_name',all_name);
	   call symput('avg_name',avg_name);	   
       call symput('std_name',std_name);
	   call symput('max_name',max_name);
	   call symput('min_name',min_name);
	   call symput('sm_name',sm_name);
   end;
run; 
%put &all_name. &avg_name. &std_name. &max_name. &min_name.  &sm_name. ;

PROC MEANS data=myLib.&inputname MIN MAX MEAN STD KURTOSIS SKEWNESS;
   var &all_name.;
run;

PROC FREQ DATA=myLib.&inputname;
	TABLES &all_name.;
RUN;

******begin: Fleishman's approache******;
/*step 1: Use the Fleishman transform to get the coefficient for the given skew and kurtosis. 
Fleishman takes the skew and kurtosis and gives you the coefficients. The coefficients was obtained using Mathematica.
For example: 
skewness=1.5, kurtoisis = 25. 
a = -0.201827, b = 1.50632, c = 0.201827, d = -0.328259*/
*******LH******;
/*%let a=0; %let b=1.14636; %let c=0; %let d=-0.4071;*/
*******HH******;
/*%let a=-0.341454; %let b=0.829293; %let c=0.341454; %let d=-0.346916;*/
*******LL******;
%let a=0; %let b=1; %let c=0; %let d=0;
			
/*step 2: Generate normal variables (mean = 0, std = 1)*/
Data work.SimulatedData;
	do ID=1 to &numRecords;
		*generate random product quantity;
		Prod_Quant_A=rand("normal", 0, 1);
		Prod_Quant_B=rand("normal", 0, 1);
		Prod_Quant_C=rand("normal", 0, 1);
		Prod_Quant_D=rand("normal", 0, 1);
		Prod_Quant_E=rand("normal", 0, 1);
		Prod_Quant_F=rand("normal", 0, 1);
		Prod_Quant_G=rand("normal", 0, 1);
		Prod_Quant_H=rand("normal", 0, 1);
		output;
	end;
run;
PROC MEANS data=work.SimulatedData MIN MAX MEAN STD KURTOSIS SKEWNESS;
run;

/* step 3: Transform the data in step 2 with the Fleishman 
coefficients to transform the normal data to the given 
skew and kurtosis: Y = a + bX + cX^2 + dX^3 */
Data work.SimulatedData;
set work.SimulatedData;
drop Prod_Quant_A Prod_Quant_B Prod_Quant_C Prod_Quant_D Prod_Quant_E
	 Prod_Quant_F Prod_Quant_G Prod_Quant_H;
Prod_Quant_A_=&a + &b*Prod_Quant_A + &c*Prod_Quant_A**2 + &d*Prod_Quant_A**3;
Prod_Quant_B_=&a + &b*Prod_Quant_B + &c*Prod_Quant_B**2 + &d*Prod_Quant_B**3;
Prod_Quant_C_=&a + &b*Prod_Quant_C + &c*Prod_Quant_C**2 + &d*Prod_Quant_C**3;
Prod_Quant_D_=&a + &b*Prod_Quant_D + &c*Prod_Quant_D**2 + &d*Prod_Quant_D**3;
Prod_Quant_E_=&a + &b*Prod_Quant_E + &c*Prod_Quant_E**2 + &d*Prod_Quant_E**3;
Prod_Quant_F_=&a + &b*Prod_Quant_F + &c*Prod_Quant_F**2 + &d*Prod_Quant_F**3;
Prod_Quant_G_=&a + &b*Prod_Quant_G + &c*Prod_Quant_G**2 + &d*Prod_Quant_G**3;
Prod_Quant_H_=&a + &b*Prod_Quant_H + &c*Prod_Quant_H**2 + &d*Prod_Quant_H**3;
run; 
PROC MEANS data=work.SimulatedData MIN MAX MEAN STD KURTOSIS SKEWNESS;
run;

/*Step 4: In this step, use data from step 3 and transform it to the desired mean 
and standard deviation (std) using new_data = desired mean + (data from step 3)* desired std*/
%let tMean = 4;
%let tStd = 50;
Data work.SimulatedData;
set work.SimulatedData;
drop Prod_Quant_A_ Prod_Quant_B_ Prod_Quant_C_ Prod_Quant_D_
     Prod_Quant_E_ Prod_Quant_F_ Prod_Quant_G_ Prod_Quant_H_;
Prod_Quant_A=floor(&tMean + Prod_Quant_A_*&tStd);
Prod_Quant_B=floor(&tMean + Prod_Quant_B_*&tStd);
Prod_Quant_C=floor(&tMean + Prod_Quant_C_*&tStd);
Prod_Quant_D=floor(&tMean + Prod_Quant_D_*&tStd);
Prod_Quant_E=floor(&tMean + Prod_Quant_E_*&tStd);
Prod_Quant_F=floor(&tMean + Prod_Quant_F_*&tStd);
Prod_Quant_G=floor(&tMean + Prod_Quant_G_*&tStd);
Prod_Quant_H=floor(&tMean + Prod_Quant_H_*&tStd);
run; 
PROC MEANS data=work.SimulatedData MIN MAX MEAN STD KURTOSIS SKEWNESS;
run;
******end: Fleishman's approache******;

/*Simulating Data Step 4: Aissgin percentages of zeros to each variable*/
%let pct=0.5;
Data work.SimulatedData;
set work.SimulatedData;
drop i1 i2 i3 i4 i5 i6 i7 i8;
*i1=rand("UNIFORM"); *if i1<=&pct or Prod_Quant_A<0 then Prod_Quant_A=0;
i1=rand("UNIFORM"); if i1<=&pct then Prod_Quant_A=0;
i2=rand("UNIFORM"); if i2<=&pct then Prod_Quant_B=0;
i3=rand("UNIFORM"); if i3<=&pct then Prod_Quant_C=0;
i4=rand("UNIFORM"); if i4<=&pct then Prod_Quant_D=0;
i5=rand("UNIFORM"); if i5<=&pct then Prod_Quant_E=0;
i6=rand("UNIFORM"); if i6<=&pct then Prod_Quant_F=0;
i7=rand("UNIFORM"); if i7<=&pct then Prod_Quant_G=0;
i8=rand("UNIFORM"); if i8<=&pct then Prod_Quant_H=0;
run;

PROC MEANS data=work.SimulatedData MIN MAX MEAN STD KURTOSIS SKEWNESS;
run;

*summarize the final percentages of zeros in each variable;
data _null_;
    set work.SimulatedData end = eof;
    if Prod_Quant_A = 0 then Pcnt_A+1 ;
	if Prod_Quant_B = 0 then Pcnt_B+1 ;
	if Prod_Quant_C = 0 then Pcnt_C+1 ;
	if Prod_Quant_D = 0 then Pcnt_D+1 ;
	if Prod_Quant_E = 0 then Pcnt_E+1 ;
	if Prod_Quant_F = 0 then Pcnt_F+1 ;
	if Prod_Quant_G = 0 then Pcnt_G+1 ;
	if Prod_Quant_H = 0 then Pcnt_H+1 ; 
    if eof then do;
	    call symput('Pcnt_A',Pcnt_A/&numRecords);
	    call symput('Pcnt_B',Pcnt_B/&numRecords);
	    call symput('Pcnt_C',Pcnt_C/&numRecords);
		call symput('Pcnt_D',Pcnt_D/&numRecords);
		call symput('Pcnt_E',Pcnt_E/&numRecords);
		call symput('Pcnt_F',Pcnt_F/&numRecords);
		call symput('Pcnt_G',Pcnt_G/&numRecords);
		call symput('Pcnt_H',Pcnt_H/&numRecords);
   end;
run;
%put &Pcnt_A.;
%put &Pcnt_B.;
%put &Pcnt_C.;
%put &Pcnt_D.;
%put &Pcnt_E.;
%put &Pcnt_F.;
%put &Pcnt_G.;
%put &Pcnt_H.;

*copy to mylib;
Data myLib.SimulatedDataLL05;
set work.SimulatedData;
run;
